// ====================================================================
//  Class: BonusPack.MutantMessage
//
//  For displaying localized messages related to the Mutant game type.
//
//  Written by James Golding
//  (c) 2002, Epic Games, Inc.  All Rights Reserved
// ====================================================================

#exec OBJ LOAD FILE=GameSounds.uax

class MutantMessage extends LocalMessage;

var	localized string 	YouAreMutantMessage;
var			  sound		YouAreMutantSound;

var localized string 	SomeoneIsMutantMessage;
var localized string 	SomeoneIsMutantMessageTrailer;
var			  sound		SomeoneIsMutantSound;

var localized string    FFAMessage;
var			  sound		FFASound;

var localized string    BottomFeederMessage;
var			  sound		BottomFeederSound;

var localized string    NotBottomFeederMessage;
var			  sound		NotBottomFeederSound;

static function string GetString(
	optional int SwitchNum,
	optional PlayerReplicationInfo RelatedPRI_1, 
	optional PlayerReplicationInfo RelatedPRI_2,
	optional Object OptionalObject 
	)
{
	switch(SwitchNum)
	{
	case 0: // You are the Mutant
		return Default.YouAreMutantMessage;

	case 1: // Someone is the Mutant
		if (RelatedPRI_1 == None)
			 return "";

		return Default.SomeoneIsMutantMessage@RelatedPRI_1.PlayerName@Default.SomeoneIsMutantMessageTrailer;

	case 2: // First blood becomes Mutant!
		return Default.FFAMessage;

	case 3: // You are the Bottom Feeder!
		return Default.BottomFeederMessage;

	case 4: // No longer Bottom Feeder!
		return Default.NotBottomFeederMessage;
	}
}

static simulated function ClientReceive( 
	PlayerController P,
	optional int SwitchNum,
	optional PlayerReplicationInfo RelatedPRI_1, 
	optional PlayerReplicationInfo RelatedPRI_2,
	optional Object OptionalObject
	)
{
	Super.ClientReceive(P, SwitchNum, RelatedPRI_1, RelatedPRI_2, OptionalObject);

	switch(SwitchNum)
	{
	case 0: // You are the Mutant
		P.PlayAnnouncement(Default.YouAreMutantSound, 1, true);
		break;

	case 1: // Someone is the Mutant
		P.PlayAnnouncement(Default.SomeoneIsMutantSound, 1, true);
		break;

	case 2: // First blood becomes Mutant!
		P.PlayAnnouncement(Default.FFASound, 1, true);
		break;

	case 3: // You are the Bottom Feeder!
		P.PlayAnnouncement(Default.BottomFeederSound, 1, true);
		break;

	case 4: // No longer Bottom Feeder!
		P.PlayAnnouncement(Default.NotBottomFeederSound, 1, true);
		break;
	}
}

defaultproperties
{
	bFadeMessage=True
	bIsSpecial=True
	bIsUnique=True
	Lifetime=6
	bBeep=False
    DrawColor=(R=255,G=0,B=128,A=255)

	YouAreMutantMessage="You Have Mutated!"
	YouAreMutantSound=sound'AnnouncerEvil.you_have_mutated'

	SomeoneIsMutantMessage=" "
	SomeoneIsMutantMessageTrailer="Has Mutated!"
	SomeoneIsMutantSound=sound'AnnouncerEvil.new_mutant'

	FFAMessage="First Blood Mutates!"
	FFASound=sound'GameSounds.UT2K3Fanfare11'

	BottomFeederMessage="You Are The Bottom Feeder!"
	BottomFeederSound=sound'AnnouncerEvil.bottom_feeder'

	NotBottomFeederMessage="No Longer Bottom Feeder!"
	NotBottomFeederSound=sound'GameSounds.UT2K3Fanfare08'

	StackMode=SM_Down
    PosY=0.25
}