// ====================================================================
// (C) 2002, Epic Games
// ====================================================================

class UT2OGGMenu extends UT2MusicManager;

var GUIListbox Songs;		// The current Play List
Var GUIListBox SongsPL;		// All Songs available
var GUILabel SongTitle;		// Title of the current Song
var GUISlider Volume;		// Volume control slider

var bool bInitialized;		// Has this menu been initialized yet

var string NextTrack;
var OGGMusicChanger Changer;

// Hack: don't use PlayList.Songs as there is a config length problem.
var config array<string>	PlayListSongs;

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	local array<string> OGGList;
    local int i,j;

 	Super.InitComponent(MyController, MyOwner);

    if (bInitialized)		// This is a presistent form, don't keep reiniting it
    {
    	if (PlayerOwner()!=None)
        {
	    	Volume.SetValue(float(PlayerOwner().ConsoleCommand("get ini:Engine.Engine.AudioDevice MusicVolume")));
			SongTitle.Caption = PlayerOwner().Level.Song;
        }

    	return;
    }

	// Setup links

    Songs   	= GUIListBox(Controls[1]);
    SongsPL 	= GUIListBox(Controls[6]);
    SongTitle 	= GUILabel(Controls[5]);
    Volume  	= GUISlider(Controls[8]);

	// Setup Delegates

	Songs.List.OnDblClick 	= SongChange;
	SongsPL.List.OnDblClick = SongsDblClick;

	// Set Defaults

    Controller.GetOGGList(OGGList);	// Pull all OGG files from the MUSIC directory

	// Add them to Play list removing their extension

	for (i=0;i<OGGList.Length;i++)
    {
    	SongsPL.List.Add(left( OGGList[i],len(OGGList[i])-4));
    }

	// Get the current song being played

	// Remove all songs from the current player list from the master list

	for (i=0;i<PlayListSongs.Length;i++)
    {
        Songs.List.Add(PlayListSongs[i]);
        for (j=0;j<SongsPL.List.ItemCount;j++)
        {
            if (PlayListSongs[i]~=SongsPL.List.Elements[j].Item)
                SongsPL.List.Remove(j,1);
        }
    }

	moCheckBox(Controls[11]).Checked(PlayList.bRepeat);
	moCheckBox(Controls[12]).Checked(PlayList.bShuffle);

	SongTitle.Caption = PlayList.Current;

    bInitialized = true;

    Changer = OGGMusicChanger(Controller.Master.AddInteraction("OGGPlayer.OGGMusicChanger"));
    if (Changer==None)
    	log("Could not spawn OGG Changer Interaction");
    else
    	Changer.Manager = self;

}

delegate OnClose(optional Bool bCanceled)
{
    SaveConfig();
}

function VolumeChange(GUIComponent Sender)
{
	PlayerOwner().ConsoleCommand("set ini:Engine.Engine.AudioDevice MusicVolume"@Volume.Value);
}

function SaveSongList()
{
    local int i;

   	if (PlayListSongs.Length>0)
      	PlayListSongs.Remove(0,PlayListSongs.Length);

    for (i=0;i<Songs.List.ItemCount;i++)
       	PlayListSongs[i] = Songs.List.Elements[i].Item;
}

function bool SongChange(GUIComponent Sender)
{
	local Actor.EMusicTransition	Transition;

	if (!bInitialized)
    	return false;

	SaveSongList();

    Transition = MTRAN_Instant;
	PlayerOwner().ClientSetMusic( Songs.List.SelectedText(),Transition);
	SongTitle.Caption = Songs.List.SelectedText();

    return true;

}

function RepeatChange(GUIComponent Sender)
{
	PlayList.bRepeat = moCheckBox(Sender).IsChecked();
}

function ShuffleChange(GUIComponent Sender)
{
	PlayList.bShuffle = moCheckBox(Sender).IsChecked();
}


function bool AddAll(GUIComponent Sender)
{
	Songs.List.LoadFrom(SongsPL.List,false);
	SongsPL.List.Clear();
    SaveSongList();
    return true;
}

function bool RemoveAll(GUIComponent Sender)
{
	SongsPL.List.LoadFrom(Songs.List,false);
	Songs.List.Clear();
    SongsPL.List.SortList();
    SaveSongList();
    return true;
}

function bool Remove(GUIComponent Sender)
{
    if (Songs.List.ItemCount>0 && Songs.List.SelectedText()!="")
    {
        SongsPL.List.Add(Songs.List.SelectedText());
        Songs.List.Remove(Songs.List.Index,1);
    }

    SaveSongList();
	SongsPL.List.SortList();

    return true;
}

function bool Add(GUIComponent Sender)
{
	local string Song;
	local Actor.EMusicTransition	Transition;

   	if (SongsPL.List.ItemCount>0 && SongsPL.List.SelectedText()!="")
    {
    	Song = SongsPL.List.SelectedText();
        Songs.List.Add(SongsPL.List.SelectedText());
        SongsPL.List.Remove(SongsPL.List.Index,1);

		SetMusic(Song);
	    Transition = MTRAN_Instant;
		PlayerOwner().ClientSetMusic( Song,Transition);
		SongTitle.Caption = Song;

    }

    SaveSongList();
	SongsPL.List.SortList();

    return true;
}

function bool SongsDblClick(GUIComponent Sender)
{
	Add(Sender);
    return true;
}

function bool UpClick(GUIComponent Sender)
{
	local int i;
	if ( Songs.List.SelectedText()=="" )
    	return true;

	i = Songs.List.Index;
    if (i>0)
    {
    	Songs.List.Swap(i,i-1);
	    Songs.List.Index = i-1;
    }
    SaveSongList();
    SaveConfig();

	return true;
}

function bool DownClick(GUIComponent Sender)
{
	local int i;
	if ( Songs.List.SelectedText()=="" )
    	return true;

	i = Songs.List.Index;
    if (i<Songs.List.ItemCount-1)
    {
    	Songs.List.Swap(i,i+1);
	    Songs.List.Index = i+1;
    }

    SaveSongList();
    SaveConfig();
	return true;
}

function SetMusic(string NewSong)
{
	Super.SetMusic(NewSong);
	SongTitle.Caption = PlayList.Current;
    SaveConfig();
}

function String SetInitialMusic(string NewSong)
{
	local int i;
	if (PlayListSongs.Length==0)
		return NewSong;

    for (i=0;i<PlayListSongs.Length;i++)
    	if (PlayListSongs[i]~=NewSong)
        	return NewSong;

   	for (i=0;i<PlayListSongs.Length;i++)
	   	if (PlayListSongs[i]~=PlayList.Current)
         	return PlayList.Current;

   	return PlayListSongs[0];
}

function NewTrack()
{
	PlayerOwner().ClientSetMusic(NextTrack,MTRAN_Fade);
}

function MusicChanged()
{
	local int i,index;

	if (PlayListSongs.Length==0)	// Nothing in play list, loop current
    	return;

	if (PlayList.bShuffle)
    {
    	i = rand(PlayListSongs.Length);
        NextTrack = PlayListSongs[i];
        Changer.bRequiresTick = true;
//        PlayerOwner().ClientSetMusic(PlayListSongs[i],MTRAN_Fade);
	}
    else
    {
		Index=-1;
		for (i=0;i<PlayListSongs.Length;i++)
        {
        	if (PlayListSongs[i] ~= PlayList.Current)
				Index = i;
        }

       	Index++;
        if (Index>=PlayListSongs.Length)
        {
        	if (PlayList.bRepeat)
		       	Index = 0;
            else
            {
            	NextTrack = "";
                Changer.bRequiresTick=true;
//                PlayerOwner().ClientSetMusic("",MTRAN_Fade);
                return;
            }
        }
      	NextTrack = PlayListSongs[Index];
        Changer.bRequiresTick=true;

//		PlayerOwner().ClientSetMusic(PlayListSongs[index],MTRAN_Fade);
    }
}



defaultproperties
{

	Begin Object Class=GUIImage name=OGGBackground
		bAcceptsInput=false
		bNeverFocus=true
		Image=Material'InterfaceContent.Menu.EditBox'
        ImageStyle=ISTY_Stretched
		WinWidth=0.962500
		WinHeight=0.870313
		WinLeft=0.014063
		WinTop=0.058333
        bBoundToParent=true
        bScaleToParent=true
	End Object
	Controls(0)=GUIImage'OGGBackground'

	Begin Object Class=GUIListBox Name=OGGSongs
		WinWidth=0.907421
		WinHeight=0.273009
		WinLeft=0.037501
		WinTop=0.160416
		bVisibleWhenEmpty=true
		StyleName="SquareButton"
        bBoundToParent=true
        bScaleToParent=true
	End Object
	Controls(1)=GUIListBox'OGGSongs'

	Begin Object Class=GUIImage name=OGGBackground2
		bAcceptsInput=false
		bNeverFocus=true
		Image=Material'InterfaceContent.Menu.EditBox'
        ImageStyle=ISTY_Stretched
		WinWidth=0.767187
		WinHeight=0.057813
		WinLeft=0.178126
		WinTop=0.075000
        bBoundToParent=true
        bScaleToParent=true
	End Object
	Controls(2)=GUIImage'OGGBackground2'

	Begin Object class=GUIButton Name=OGGNextTrack
		WinWidth=0.043555
		WinHeight=0.084414
		WinLeft=0.294531
		WinTop=0.894793
		bNeverFocus=true
		bRepeatClick=true
		OnClickSound=CS_Down
		StyleName="ArrowRight"
        bBoundToParent=true
        bScaleToParent=true
        bVisible=false
	End Object
	Controls(3)=GUIButton'OGGNextTrack'

	Begin Object class=GUILabel Name=OGGNowPlaying
		Caption="Now Playing"
		TextALign=TXTA_Center
		TextColor=(R=255,G=255,B=255,A=255)
		WinWidth=0.165625
		WinHeight=32.000000
		WinLeft=0.022851
		WinTop=0.070833
	End Object
	Controls(4)=GUILabel'OGGNowPlaying'

	Begin Object class=GUILabel Name=OGGSongTitle
		Caption=""
		TextALign=TXTA_Left
		TextColor=(R=255,G=255,B=0,A=255)
		WinWidth=0.735937
		WinHeight=32.000000
		WinLeft=0.193164
		WinTop=0.070833
	End Object
	Controls(5)=GUILabel'OGGSongTitle'

	Begin Object Class=GUIListBox Name=OGGSongsPL
		WinWidth=0.907421
		WinHeight=0.273009
		WinLeft=0.037501
		WinTop=0.535416
		bVisibleWhenEmpty=true
		StyleName="SquareButton"
        bBoundToParent=true
        bScaleToParent=true
	End Object
	Controls(6)=GUIListBox'OGGSongsPL'

	Begin Object class=GUILabel Name=OGGMusicVolumeLabel
		Caption="Music Volume"
		TextALign=TXTA_Center
		TextColor=(R=255,G=255,B=255,A=255)
		WinWidth=0.465625
		WinHeight=32.000000
		WinLeft=0.096289
		WinTop=0.814582
        bBoundToParent=true
        bScaleToParent=true
	End Object
	Controls(7)=GUILabel'OGGMusicVolumeLabel'

	Begin Object class=GUISlider Name=OGGMusicVolumeSlider
		WinWidth=0.498438
		WinLeft=0.079687
		WinTop=0.858228
		MinValue=0
		MaxValue=1
		Hint="Changes the volume of the background music."
        bBoundToParent=true
        bScaleToParent=true
        OnChange=VolumeChange
	End Object
	Controls(8)=GUISlider'OGGMusicVolumeSlider'

	Begin Object class=GUIButton Name=OGGAddAll
		WinWidth=0.176368
		WinHeight=0.046914
		WinLeft=0.041406
		WinTop=0.488542
		bNeverFocus=true
		bRepeatClick=true
		OnClick=AddAll;
		OnClickSound=CS_Down
        bBoundToParent=true
        bScaleToParent=true
        Caption="Add All"
	End Object
	Controls(9)=GUIButton'OGGAddAll'

	Begin Object class=GUIButton Name=OGGClear
		WinWidth=0.174805
		WinHeight=0.046914
		WinLeft=0.222656
		WinTop=0.488542
		bNeverFocus=true
		bRepeatClick=true
		OnClick=RemoveAll;
		OnClickSound=CS_Down
        bBoundToParent=true
        bScaleToParent=true
        Caption="Remove All"
	End Object
	Controls(10)=GUIButton'OGGClear'

	Begin Object class=moCheckBox Name=OGGRepeat
		WinWidth=0.093750
		WinHeight=0.040000
		WinLeft=0.666562
		WinTop=0.851719
		Caption="Repeat"
		CaptionWidth=0.9
		bSquare=true
		ComponentJustification=TXTA_Left
        OnChange=RepeatChange
   	End Object
	Controls(11)=moCheckbox'OGGRepeat'

	Begin Object class=moCheckBox Name=OGGShuffle
		WinWidth=0.100000
		WinHeight=0.040000
		WinLeft=0.810938
		WinTop=0.851719
		Caption="Shuffle"
		CaptionWidth=0.9
		bSquare=true
		ComponentJustification=TXTA_Left
        OnChange=ShuffleChange
	End Object
	Controls(12)=moCheckbox'OGGShuffle'

	Begin Object class=GUIButton Name=OGGUp
		WinWidth=0.124805
		WinHeight=0.046914
		WinLeft=0.651406
		WinTop=0.434377
		bNeverFocus=true
		bRepeatClick=true
		OnClickSound=CS_Down
        OnClick=UpClick
        bBoundToParent=true
        bScaleToParent=true
        Caption="Up"
	End Object
	Controls(13)=GUIButton'OGGUp'

	Begin Object class=GUIButton Name=OGGDown
		WinWidth=0.124805
		WinHeight=0.046914
		WinLeft=0.779531
		WinTop=0.434376
		bNeverFocus=true
		bRepeatClick=true
		OnClickSound=CS_Down
        bBoundToParent=true
        bScaleToParent=true
        OnClick=DownClick
        Caption="Down"
	End Object
	Controls(14)=GUIButton'OGGDown'


	Begin Object class=GUIButton Name=OGGAdd
		WinWidth=0.176368
		WinHeight=0.046914
		WinLeft=0.041406
		WinTop=0.436459
		bNeverFocus=true
		bRepeatClick=true
		OnClick=Add;
		OnClickSound=CS_Down
        bBoundToParent=true
        bScaleToParent=true
        Caption="Add"
	End Object
	Controls(15)=GUIButton'OGGAdd'

	Begin Object class=GUIButton Name=OGGRemove
		WinWidth=0.174805
		WinHeight=0.046914
		WinLeft=0.222656
		WinTop=0.436459
		bNeverFocus=true
		bRepeatClick=true
		OnClick=Remove;
		OnClickSound=CS_Down
        bBoundToParent=true
        bScaleToParent=true
        Caption="Remove"
	End Object
	Controls(16)=GUIButton'OGGRemove'



	bRequire640x480=false
	bAllowedAsLast=true
    WinLeft=0
    WinWidth=1
    WinTop=0
    WinHeight=1
}